# -*- coding: UTF-8 -*-

import logging
from . import models
from django.conf import settings
from importlib import import_module
from django.contrib.sessions.models import Session
from django.utils.deprecation import MiddlewareMixin

logger = logging.getLogger(__name__)

class GlobalRequestMiddleware(MiddlewareMixin):
	__instance = None
	
	def __new__(cls, *args, **kwargs):
		if not cls.__instance:
			cls.__instance = object.__new__(cls)
		return cls.__instance
	#end new
	
	def process_request(self, request):
		logger.debug("GlobalRequestMiddleware.process_request")
		GlobalRequestMiddleware.__instance = request
	#end process_request
	
	@classmethod
	def getRequest(cls):
		return cls.__instance
	#end getRequest
#end - GlobalRequestMiddleware

class PreventConcurrentLoginsMiddleware(MiddlewareMixin):
	engine = import_module(settings.SESSION_ENGINE)
	
	def process_request(self, request):
		if request.user.is_authenticated:
			key_from_cookie = request.session.session_key
			if getattr(request.user, "login_limiter", None):
				session_key_in_visitor_db = request.user.login_limiter.session_key
				ip_address_in_visitor_db = request.user.login_limiter.ip_address
				ip_address = request.META.get("HTTP_X_FORWARDED_FOR") or request.META.get("REMOTE_ADDR")
				if session_key_in_visitor_db != key_from_cookie and not request.user.has_perm("ako.concurrent_logins"):
					self.engine.SessionStore(session_key_in_visitor_db).delete()
					request.user.login_limiter.session_key = key_from_cookie
					request.user.login_limiter.ip_address = ip_address
					request.user.login_limiter.save()
				if ip_address_in_visitor_db != ip_address:
					logger.info("用户 {} 当前IP地址 {} 与上次IP地址 {} 不同".format(request.user.username, ip_address, ip_address_in_visitor_db))
			else:
				models.LoginLimiter.objects.create(
					 user = request.user,
					 session_key = key_from_cookie,
					 ip_address = request.META.get("HTTP_X_FORWARDED_FOR") or request.META.get("REMOTE_ADDR")
				).save()
	#end process_request
#end - PreventConcurrentLoginsMiddleware



# -*- coding: UTF-8 -*-

import functools

# import pydantic
from django.conf import settings
from django.db import models as d_models
from django.http import HttpResponse, JsonResponse
from django.views import View
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from xj_dictionary.services.config_service import dictionary_configure_list
from ..models import Configure
from ..services import config_service
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper


def result_json(func):
    def convert_model(v):
        dv = {}
        for f in v._meta.fields:
            fv = getattr(v, f.name)
            if isinstance(f, (d_models.ImageField, d_models.FileField)):
                if str(fv):
                    fv = "{}{}".format(settings.MEDIA_URL, fv)
                else:
                    fv = None
            if not isinstance(fv, d_models.Model):
                dv[f.name] = fv
        return dv

    def serializer(d):
        iter = None
        if isinstance(d, dict):
            iter = d.items()
        elif isinstance(d, list):
            iter = enumerate(d)
        if iter:
            for k, v in iter:
                if isinstance(v, dict) or isinstance(v, list):
                    d[k] = serializer(v)
                elif isinstance(v, (str, int, float, bool, type(None))):
                    pass
                elif isinstance(v, d_models.Model):
                    d[k] = convert_model(v)
                else:
                    d[k] = [convert_model(f) for f in v]
            # end if
        # end for
        # end if

        return d

    # end serializer

    @functools.wraps(func)
    def warpper(*args, **kwargs):
        result = serializer(func(*args, **kwargs))
        if isinstance(result, dict):
            return JsonResponse(result, safe=False, json_dumps_params={"ensure_ascii": False})
        elif isinstance(result, str):
            return HttpResponse(result)
        return result

    # end warpper

    return warpper


class DictionaryConfigure(APIView):
    # class GetResponse(pydantic.BaseModel):
    #     status: str
    #     data: typing.Any

    @request_params_wrapper
    def get(self, *args, request_params={}, **kwargs):
        """
        获取系统配置
        """
        # print("args:", args, "request_params:", request_params, "kwargs:", kwargs)
        group = request_params.get("group", None) or kwargs.get("group", None)
        key = request_params.get("key", None) or kwargs.get("key", None)
        # print("group:", group, "key", key)
        if group is None:
            return util_response(err=0, msg="group 不能为空")
        result = config_service.getConfig(group, key)
        return util_response(result, 0, status.HTTP_200_OK)

    # 添加或者修改
    @request_params_wrapper
    def post(self, *args, request_params={}, **kwargs):
        key = request_params.get('key', None) or kwargs.get("key", None)
        group = request_params.get('group', None) or kwargs.get("group", None)
        value = request_params.get('value', None) or kwargs.get("value", None)

        description = request_params.get('description', "")
        if key is None or group is None or value is None:
            return util_response(err=2055, msg="参数错误")
        config_service.setConfig(group, key, value, description)
        return util_response()

    # 删除
    def delete(self, request, key1=None):
        return config_service.delConfig(request)


class DictionaryConfigurePage(APIView):

    @api_view(["GET", "POST"])
    @request_params_wrapper
    def configure_page(self, *args, request_params=None, **kwargs):
        """
        配置综合查询列表
        :param request_params:
        :return: response
        """
        if request_params is None:
            request_params = {}
        filter_fields = request_params.pop("filter_fields", None)
        try:
            need_pagination = int(request_params.pop("need_pagination", 1))
        except Exception:
            need_pagination = 1

        data, err = dictionary_configure_list(
            filter_fields_params=request_params,
            filter_fields=filter_fields,
            need_pagination=need_pagination
        )
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)


class BatchDictionaryConfigure(View):
    def get(self, request):
        """
        批量分组展示
        :param request:
        :return: response
        """
        groups = request.GET.get('groups', None)
        if groups is None:
            return util_response(None, 7568, status.HTTP_200_OK, '参数错误')
        groups = groups.split(",")
        res_set = list(Configure.objects.filter(group__in=groups).values())
        return util_response(data=res_set, msg="该方法不在维护，推荐使用list接口")



# -*- coding: UTF-8 -*-

# from ako import router
# from . import models, forms
from django.contrib import admin

from . import models

@admin.register(models.Configure)
class ConfigureAdmin(admin.ModelAdmin):
    fields = ["group", "description", "key",  "value", 'icon']
    
    # fieldsets = [
    #     (None, {
    #         "fields": ["group", "description", "key", "value", ],
    #     }),
    # ]
    
    list_display = ["description", "group", "key", "value_short", 'icon']
    list_filter = ["group"]
    
    def get_form(self, request, obj=None, **kwargs):
        # if not router.isMobile(request):
        #     kwargs["form"] = forms.py.ConfigAdminForm
        
        return super().get_form(obj, **kwargs)
    #end get_form
#end - ConfigAdmin

# @admin.register(models.Advert)
# class AdvertAdmin(admin.ModelAdmin):
#     fieldsets = [
#         (None, {
#             "fields" : ["location", "image", "url", "hint", "expire", "sort"],
#         }),
#     ]
#
#     list_display = [ "title", "hintShort", "expire", "location", "sort" ]
#     list_filter = [ "location" ]
#
#     def hintShort(self, obj):
#         if len(obj.hint) > 32:
#             return obj.hint[:32] + "..."
#         return obj.hint
#     hintShort.short_description = "文本"
#     #end hintShort
# #end - AdvertAdmin

admin.site.site_header = 'msa一体化管理后台'
admin.site.site_title = 'msa一体化管理后台'



# -*- coding: UTF-8 -*-

from django.apps import AppConfig


class DictionaryConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'xj_dictionary'
    verbose_name = u'Ⅶ 字典模块'
    sort = 7



# -*- coding: UTF-8 -*-

from . import models
from django import forms
# from mdeditor.fields import MDTextFormField

class ConfigAdminForm(forms.ModelForm):
	class Meta:
		models = models.Configure
		fields = [ "group", "key", "value", "description" ]
	#end - Meta
	
	# value = MDTextFormField(label="参数值")
#end - ConfigAdminForm



# -*- coding: UTF-8 -*-

# import datetime
# from ako import router
# from . import servicefrom django.conf import settings
# from django.utils import timezone
from django.db import models

from .services import config_service


# Create your models here.

class Configure(models.Model):
    group = models.CharField(verbose_name="分组", max_length=128, db_index=True)
    key = models.CharField(verbose_name="参数名", max_length=128, db_index=True)
    value = models.TextField(verbose_name="参数值", blank=True, default="")
    description = models.CharField(verbose_name="说明", max_length=255)
    icon = models.ImageField(verbose_name='图标', blank=True, null=True, upload_to='static/images/%Y%m%d')

    SEO_FIELDS = ["group", "key", "value", "description"]

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        result = super().save(force_insert, force_update, using, update_fields)
        config_service.updateConfig(self.group, self.key, self.value)
        return result

    # end save

    def delete(self, using=None, keep_parents=False):
        result = super().delete(using, keep_parents)
        # utility.updateConfig()    # 完全更新缓存
        return result

    # end delete

    def value_short(self):
        if len(self.value) > 10:
            return self.value[:10] + "..."
        return self.value

    value_short.short_description = "参数值"
    value_short.admin_order_field = "value"

    # end title

    class Meta:
        db_table = 'dictionary_configure'
        verbose_name_plural = "1. 字典 - 系统配置"
        verbose_name = "配置"
        unique_together = [("group", "key")]

    # end - Meta

    def __str__(self):
        return "[{}/{}] {}".format(self.group, self.key, self.description)


























    # end str
# end - Config

# def advert_image_upload_to(instance, filename):
#     now = timezone.now()
#     db = settings.DATABASE_MAPPING.get(router.getSiteDomain())
#     if db:
#         return "{}/fdbimg/{}{}.{}".format(db, now.strftime("%Y-%m-%d/%H%M%S"), abs(hash(filename)), filename[filename.rfind(".")+1:])
#     return "fdbimg/{}{}.{}".format(now.strftime("%Y-%m-%d/%H%M%S"), abs(hash(filename)), filename[filename.rfind(".")+1:])

# end - advert_image_upload_to


# class Advert(models.Model):
#     LOCATION_CHOICES = [
#         ("pre_nav", "导航条上方(最顶)"),
#         ("pre_title", "标题上方(导航条下方)"),
#         ("pre_breadcrumb", "标题下方(面包屑上方)"),
#         ("pre_footer", "页脚上方(无页脚不显示)"),
#         ("post_footer", "页脚下方(版权信息上方)"),
#         ("carousel", "首页轮播"),
#     ]
#
#     location = models.CharField(verbose_name="位置", max_length=32, choices=LOCATION_CHOICES)
#     image = models.ImageField(verbose_name="图片", upload_to=advert_image_upload_to)
#     url = models.URLField(verbose_name="链接")
#     hint = models.CharField(verbose_name="提示文本", max_length=255, blank=True)
#     expire = models.DateTimeField(verbose_name="过期时间", db_index=True)
#     sort = models.IntegerField(verbose_name="排序", help_text="从大到小", default=0, db_index=True)
#
#     SEO_FIELDS = [ "url", "hint" ]
#
#     def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
#         result = super().save(force_insert, force_update, using, update_fields)
#         # utility.updateAdvert()
#         return result
#     #end save
#
#     def title(self):
#         return str(self)
#     title.short_description = "标题"
#     #end title
#
#     class Meta:
#         verbose_name_plural = "1.图片广告"
#         verbose_name = "广告"
#     #end - Meta
#
#     def __str__(self):
#         return "广告 {} #{}".format(self.location, self.id)
#     #end str
# #end - Advert



# encoding: utf-8
"""
@project: djangoModel->url
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/6/6 14:25
"""

from django.urls import re_path, path

from .apis.dictionary_configure import DictionaryConfigure, DictionaryConfigurePage, BatchDictionaryConfigure

# 应用名称
app_name = 'xj_dictionary'

urlpatterns = [
    path("configure/<str:group>", DictionaryConfigure.as_view()),
    path("configure/<str:group>/<str:key>", DictionaryConfigure.as_view()),
    path("configure/<str:group>/<str:key>/<str:value>", DictionaryConfigure.as_view()),
    path("configure", DictionaryConfigure.as_view()),
    re_path("^list/?$", DictionaryConfigurePage.configure_page),  # 分页展示配置
    re_path("^groups/?$", BatchDictionaryConfigure.as_view()),  # TODO 暂时保留，后面将会去除掉该接口，批量搜索功能由上面接口代替
]
